from telegram import Update, InlineKeyboardMarkup
from telegram.ext import CallbackContext

from .utils.context_logic import get_chat_data, put_chat_data
from .text_handler import text_handler
from .mylogging import logger


# Caching logic
async def register_last_active_keyboard(context: CallbackContext, message_id, user_id=None):
    put_chat_data(context, "last_keyboard_message_id", message_id, user_id)


async def mute_last_active_keyboard(update: Update, context: CallbackContext, user_id=None):
    chat_data = get_chat_data(context, user_id)
    message_id = chat_data.get("last_keyboard_message_id")
    if message_id:
        try:
            await context.bot.delete_message(
                chat_id=user_id or update.effective_chat.id,
                message_id=message_id,
            )
            chat_data.pop("last_keyboard_message_id")

        except Exception as e:
            logger.warning(f"Error muting last active keyboard {user_id}: {str(e)}")


# Send raw
async def send_text_raw(update: Update, context: CallbackContext, text: str, user_id = None, **kwargs):
    sent_message = await context.bot.send_message(chat_id=user_id or update.effective_chat.id, text=text, **kwargs)
    return sent_message.message_id


async def send_keyboard_raw(
    update: Update,
    context: CallbackContext,
    keyboard: InlineKeyboardMarkup,
    text: str,
    clear_after=True,
    clear_before = True,
    user_id=None,
    **kwargs,
):

    if clear_before:
        # Mute a previous active window so only one active window can be in a chat
        await mute_last_active_keyboard(update, context, user_id)

    # Send
    sent_message_id = await send_text_raw(
        update, context, text, user_id = user_id, reply_markup=keyboard, **kwargs
    )

    if clear_after:
        # Store the message ID of the new start menu message
        await register_last_active_keyboard(context, sent_message_id, user_id)


# text_string_index needed only
async def send_text(
    update: Update,
    context: CallbackContext,
    text_string_index: str,
    user_id = None,
    **kwargs
):
    TEXT, PARSE_MODE = text_handler.get_text_and_parse_mode(
        context, text_string_index, user_id
    )
    return await send_text_raw(
        update,
        context,
        TEXT,
        user_id=user_id,
        parse_mode=PARSE_MODE,
        # disable_web_page_preview=True,
        **kwargs,
    )


async def send_keyboard(
    update: Update,
    context: CallbackContext,
    keyboard: InlineKeyboardMarkup,
    text_string_index: str,
    user_id=None,
    clear_after=True,
    clear_before=True,
    **kwargs,
):
    TEXT, PARSE_MODE = text_handler.get_text_and_parse_mode(
        context, text_string_index, user_id
    )
    return await send_keyboard_raw(
        update,
        context,
        keyboard,
        TEXT,
        user_id=user_id,
        clear_after=clear_after,
        clear_before=clear_before,
        parse_mode=PARSE_MODE,
        disable_web_page_preview=True,
        **kwargs,
    )


async def send_video_from_file_id_raw(
    update, context, keyboard, file_id, caption, parse_mode, user_id=None, **kwargs
):
    if user_id is None:
        user_id = update.effective_chat.id

    sent_message = await context.bot.send_video(
        chat_id=user_id,
        reply_markup=keyboard,
        video=file_id,
        caption=caption,
        parse_mode=parse_mode,
        **kwargs
    )
    return sent_message.message_id


async def send_video_from_file_id(
    update,
    context,
    keyboard,
    file_id,
    caption=None,
    parse_mode=None,
    user_id=None,
    clear_after=True,
    clear_before=True,
    **kwargs,
):
    """Sends video as a keyboard"""
    # TODO the following structure as a decorator

    if clear_before:
        # Mute a previous active window so only one active window can be in a chat
        await mute_last_active_keyboard(update, context, user_id)

    # Send
    sent_message_id = await send_video_from_file_id_raw(
        update, context, keyboard, file_id, caption, parse_mode, **kwargs
    )

    if clear_after:
        # Store the message ID of the new start menu message
        await register_last_active_keyboard(context, sent_message_id, user_id)



# Stuff to use
__version__ = "0.1.0"
from .app import telegram_bot_polling
from .mylogging import time_log_decorator, logger, get_time
from .decorators import command, button_callback, register_conversation_handler
from .admin import admin, ADMIN_MENU
from .text_handler import text_handler
from .send import send_text, send_keyboard
from .utils.utils import (
    get_keyboard,
    get_info_from_query,
    end_conversation,
)
from .validate_text_file import clear_table_element
from .file_handler import FileHandler
from .video_handler import VideoFileHandler
# for adding the handlers on init automatically
from .start import start_conv_handler
from .put_file_conv import PutFileConversation
from .put_intro_video_conv import PutVideoConversation
__version__ = "0.1.1"
from .utils.context_logic import get_chat_data, put_chat_data, get_user_data, put_user_data, clear_cache
from .send import send_video_from_file_id
from .roles import role_required, check_role, DEFAULT_ADMIN_ROLES, DEFAULT_ALLOWED_ROLES, get_people_layout, get_people_role_group
from .send_with_navigation import send_page_with_navigation, send_page_nav
# for adding the handlers on init automatically
from .manage_role_conv import ManageRoleConverstion
# Serverless lambda
from .app import lambda_handler
__version__ = "0.1.3"
from .decorators import message_handler
__version__ = "0.1.4"
from .app import application
from .settings import get_secret_by_name, default_roles
__version__ = "0.1.5"
# for adding the handlers on init automatically
from .mail_users_conv import message_mailing_conv_handler
from . import superadmin # INFO_LINES for superadmin menu
from . import error
__version__ = "0.1.6"
# TODO
# - Forget user /forget_me; Delete user;
# - Hung convo bug - test
# - Connection error handler
# - Limiter
# - Replace 'nan' with None
# - Small Bug fix: Error in not_data_consent ?
# - Bug fix: None.log occasionally on run
# - /restart_me
# - optional number share


import os
import shutil


# Consts
DRAFTS_FOLDER = "templates"
DRAFTS_FOLDER_PATH = os.path.join(
    os.path.dirname(__file__), DRAFTS_FOLDER
)


def initialize_file_from_draft(file_name, root_dir):
    # Create files
    output_file = os.path.join(root_dir, file_name)
    if os.path.exists(output_file):
        return

    # Debug
    if not os.path.exists(DRAFTS_FOLDER_PATH):
        print(f"Templates folder '{DRAFTS_FOLDER_PATH}' does not exist.")
        return

    draft_path = os.path.join(DRAFTS_FOLDER_PATH, file_name)
    if not os.path.exists(draft_path):
        print(f"{draft_path} does not exist.")
        return 

    # Create files
    shutil.copy(draft_path, output_file)
    print(f"{file_name} file created.")

    return True


def initialize_all_files_from_drafts(root_dir):
    return all(
        initialize_file_from_draft(file_name, root_dir)
        for file_name in os.listdir(DRAFTS_FOLDER_PATH)
    )



import contextlib
import asyncio
import json
import os

from telegram.ext import Application, ApplicationBuilder, PicklePersistence
from telegram import Update

from .utils.init_templates import initialize_file_from_draft
from .mylogging import time_log_decorator, logger
from . import settings

# add handlers just with decorators throughout the code
from .decorators import add_handlers


# -------------------------------------------TELEGRAM BOT DB------------------------------
class MyApplication(Application):
    # Disagreeing with the user/chat_data read-only policy
    def __init__(
        self,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.user_data = self._user_data
        self.chat_data = self._chat_data

    async def stop(self):
        # TODO handle urgent shutdown
        # Ensure all user data and chat data are saved
        if self.persistence:
            for user_id, data in self.user_data.items():
                await self.persistence.update_user_data(user_id, data)
            for chat_id, data in self.chat_data.items():
                # Sanitize for unpickable objects: convention
                for k in list(data.keys()):
                    if k.endswith("_func"):
                        data.pop(k)
                await self.persistence.update_chat_data(chat_id, data)

        # Call the original stop method
        await super().stop()


# --------------------------------------------TELEGRAM APP---------------------------------
def prepare_app():
    root_dir = settings.TG_FILE_FOLDER_PATH or os.getcwd() 
    initialize_file_from_draft("text.xlsx", root_dir)
    persistence = PicklePersistence(
        filepath=f"{settings.TG_FILE_FOLDER_PATH}{settings.TG_BOT_NAME}Persistence", on_flush=True
    )
    application = (
        ApplicationBuilder()
        .application_class(MyApplication)
        .token(settings.TG_BOT_TOKEN)
        .persistence(persistence)
        .build()
    )

    return application

# App instance
application = prepare_app()

# Operations
@time_log_decorator
def telegram_bot_polling(debug = False):
    # Add handlers; needs to be called at the end for the decorators reason
    add_handlers(application, debug)
    # Ignore exception when Ctrl-C is pressed
    with contextlib.suppress(KeyboardInterrupt):  
        application.run_polling()


@time_log_decorator
async def process_update(event, context):
    # Add handlers; needs to be called at the end for the decorators reason
    add_handlers(application)
    try:
        await application.initialize()
        await application.process_update(
            Update.de_json(json.loads(event["body"]), application.bot)
        )
        await application.stop()
        return {"statusCode": 200, "body": "Success"}

    except Exception as e:
        logger.error(f"Error in process_update: {e}")
        return {"statusCode": 500, "body": "Failure"}


def lambda_handler(event, context):
    return asyncio.get_event_loop().run_until_complete(process_update(event, context))



import easy_tg_bot
# initiate other modules
import settings


@easy_tg_bot.command()
async def help(update, context):
    return await easy_tg_bot.send_text(update, context, "help_message")  # text.xlsx

if __name__=="__main__":
    easy_tg_bot.start.START_DONE_CALLBACK = help
    easy_tg_bot.telegram_bot_polling()
